export declare type Subscription<T> = (value: T | undefined, prevValue: T | undefined) => void;
export declare enum ResolvePriority {
    Takes = "takes",
    Loads = "loads"
}
/**
 * A cell that holds a value and can alert users when a value in inserted.
 */
export default class AsyncCell<T> {
    private readonly priority;
    private inner?;
    private subscriptions;
    private resolvers;
    constructor(value?: T, priority?: ResolvePriority);
    /**
     * Returns the value from the cell or waits for a value to be inserted.
     * @returns the value in the cell.
     */
    load(): Promise<T>;
    /**
     * Waits for a value to be placed in the cell and then resets it's inner value.
     * @returns the value placed into the cell.
     */
    take(): Promise<T>;
    /**
     * Peeks into the cell to check the current inner value.
     * @returns the current inner value of the cell.
     */
    peek(): T | undefined;
    /**
     * Inserts a value into the cell and resolves all promises waiting on the cell to be filled.
     * @param value the value to insert into the cell.
     */
    insert(value?: T): void;
    /**
     * Registers a {@link Subscription} that is called when a new value is inserted into the cell.
     * @param subscriptions subscriptions to be invoked when a new value is inserted into the cell.
     */
    subscribe(...subscriptions: Subscription<T>[]): void;
}
