"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResolvePriority = void 0;
var ResolvePriority;
(function (ResolvePriority) {
    ResolvePriority["Takes"] = "takes";
    ResolvePriority["Loads"] = "loads";
})(ResolvePriority = exports.ResolvePriority || (exports.ResolvePriority = {}));
/**
 * A cell that holds a value and can alert users when a value in inserted.
 */
var AsyncCell = /** @class */ (function () {
    function AsyncCell(value, priority) {
        if (priority === void 0) { priority = ResolvePriority.Takes; }
        this.subscriptions = [];
        this.resolvers = {
            takes: [],
            loads: [],
        };
        this.inner = value;
        this.priority = priority;
    }
    /**
     * Returns the value from the cell or waits for a value to be inserted.
     * @returns the value in the cell.
     */
    AsyncCell.prototype.load = function () {
        var _this = this;
        if (this.inner !== undefined) {
            return Promise.resolve(this.inner);
        }
        return new Promise(function (resolve) {
            _this.resolvers.loads.push(resolve);
        });
    };
    /**
     * Waits for a value to be placed in the cell and then resets it's inner value.
     * @returns the value placed into the cell.
     */
    AsyncCell.prototype.take = function () {
        return __awaiter(this, void 0, void 0, function () {
            var oldValue;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.inner !== undefined) {
                            oldValue = this.inner;
                            this.inner = undefined;
                            return [2 /*return*/, oldValue];
                        }
                        return [4 /*yield*/, new Promise(function (resolve) {
                                _this.resolvers.takes.push(resolve);
                            })];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * Peeks into the cell to check the current inner value.
     * @returns the current inner value of the cell.
     */
    AsyncCell.prototype.peek = function () {
        return this.inner;
    };
    /**
     * Inserts a value into the cell and resolves all promises waiting on the cell to be filled.
     * @param value the value to insert into the cell.
     */
    AsyncCell.prototype.insert = function (value) {
        var _this = this;
        var prevValue = this.inner;
        if (value === undefined) {
            this.inner = undefined;
            this.subscriptions.forEach(function (subscription) {
                return subscription(undefined, prevValue);
            });
        }
        else {
            this.inner = value;
            this.subscriptions.forEach(function (subscription) {
                return subscription(value, prevValue);
            });
        }
        if (this.inner === undefined)
            return;
        // If we prioritize loads we'll resolve them before we resolve takes.
        if (this.priority === ResolvePriority.Loads) {
            this.resolvers.loads.forEach(function (resolver) { return resolver(_this.inner); });
            this.resolvers.loads = [];
        }
        var takeResolveFunction = this.resolvers.takes.shift();
        if (takeResolveFunction !== undefined) {
            takeResolveFunction(this.inner);
        }
        else if (this.priority === ResolvePriority.Takes) {
            // Otherwise if we prioritize takes we'll resolve them after we do any takes, but not if there were tany takes.
            this.resolvers.loads.forEach(function (resolver) { return resolver(_this.inner); });
            this.resolvers.loads = [];
        }
    };
    /**
     * Registers a {@link Subscription} that is called when a new value is inserted into the cell.
     * @param subscriptions subscriptions to be invoked when a new value is inserted into the cell.
     */
    AsyncCell.prototype.subscribe = function () {
        var subscriptions = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            subscriptions[_i] = arguments[_i];
        }
        this.subscriptions = __spreadArray(__spreadArray([], this.subscriptions), subscriptions);
    };
    return AsyncCell;
}());
exports.default = AsyncCell;
